﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.Diagnostics.Runtime;

namespace Winterdom.Diagnostics {
  public partial class MainForm : Form {
    private DacLocator dacloc;
    private ClrRuntime clrRuntime;

    public MainForm() {
      InitializeComponent();
      statusStrip.Text = "";
    }

    private void OnFindHttpContextButtonClick(object sender, EventArgs e) {
      CleanUp();
      dacloc = DacLocator.FromPublicSymbolServer(localCachePathTextBox.Text);
      DataTarget dt = DataTarget.LoadCrashDump(dumpFileTextBox.Text);

      statusStrip.Text = String.Format("Locating DAC library...");
      Application.DoEvents();
      String dac = dacloc.FindDac(dt.ClrVersions[0]);
      if ( String.IsNullOrEmpty(dac) ) {
        statusStrip.Text = "DAC library could not be found.";
        return;
      }
      statusStrip.Text = String.Format("Using {0}", dac);
      Application.DoEvents();

      clrRuntime = dt.CreateRuntime(dac);
      var httpCtxts = FindHttpContexts(clrRuntime);
      resultsGridView.DataSource = httpCtxts.ToList();
    }

    private IEnumerable<HttpCtxtInfo> FindHttpContexts(ClrRuntime clr) {
      ClrHeap heap = clr.GetHeap();
      foreach ( ulong addr in heap.EnumerateObjects() ) {
        ClrType type = heap.GetObjectType(addr);
        if ( type == null ) continue;
        if ( type.Name != "System.Web.HttpContext" ) continue;

        yield return GetHttpContextInfo(heap, addr, type); 
      }
    }

    private HttpCtxtInfo GetHttpContextInfo(ClrHeap heap, ulong addr, ClrType type) {
      HttpCtxtInfo info = new HttpCtxtInfo {
        Address = addr
      };
      ulong reqAddr = (ulong)type.GetFieldByName("_request").GetFieldValue(addr);
      ClrType reqType = heap.GetObjectType(reqAddr);
      info.Method = (string)reqType.GetFieldByName("_httpMethod").GetFieldValue(reqAddr);
      info.Url = (string)reqType.GetFieldByName("_rawUrl").GetFieldValue(reqAddr);
      return info;
    }

    private void CleanUp() {
      clrRuntime = null;
      if ( dacloc != null ) {
        dacloc.Dispose();
        dacloc = null;
      }
    }
  }
}
